/*  bsp.h
 *
 *  This include file contains all efi332 board IO definitions.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  bsp.h,v 1.2 1995/12/19 20:08:29 joel Exp
 */

#ifndef __COLD_BSP_h
#define __COLD_BSP_h

#ifdef __cplusplus
extern "C" {
#endif

#include <rtems.h>
#include <clockdrv.h>
#include <console.h>
#include <iosupp.h>
#include <cold.h>
#include <sim5206.h>

/*
 *  Define the time limits for RTEMS Test Suite test durations.
 *  Long test and short test duration limits are provided.  These
 *  values are in seconds and need to be converted to ticks for the
 *  application.
 *
 */

#define MAX_LONG_TEST_DURATION       300 /* 5 minutes = 300 seconds */
#define MAX_SHORT_TEST_DURATION      3   /* 3 seconds */

/*
 *  Define the interrupt mechanism for Time Test 27
 */

#if 0
/* XXX - JRS - I want to compile the tmtests */

#define MUST_WAIT_FOR_INTERRUPT 1 

#define Install_tm27_vector( handler )

#define Cause_tm27_intr()

#define Clear_tm27_intr()

#define Lower_tm27_intr() 

/*
 *  Simple spin delay in microsecond units for device drivers.
 *  This is very dependent on the clock speed of the target.
 */

#define delay( microseconds ) \
  { register rtems_unsigned32 _delay=(microseconds); \
    register rtems_unsigned32 _tmp=123; \
    asm volatile( "0: \
                     nbcd      %0 ; \
                     nbcd      %0 ; \
                     dbf       %1,0b" \
                  : "=d" (_tmp), "=d" (_delay) \
                  : "0"  (_tmp), "1"  (_delay) ); \
  }
#endif

/* macros */

#define RAM_START 0x010000
#define RAM_END   0x400000

extern void RAW_FMT( const char*, ... );

#define RAW_PUTS(str)	RAW_FMT("%s",(str))
#define RAW_PUTI(n)	RAW_FMT("%x",(n))

/* miscellaneous stuff assumed to exist */

extern rtems_configuration_table BSP_Configuration;

#if 0
extern m68k_isr_entry M68Kvec[];   /* vector table address */
#endif

extern const int stack_size[];
extern const int heap_size[];

extern int stack_start;

/*
 *  Device Driver Table Entries
 */
 
/*
 * NOTE: Use the standard Console driver entry
 */
 
/*
 * NOTE: Use the standard Clock driver entry
 */

/*
 * How many libio files we want
 */
 
#define BSP_LIBIO_MAX_FDS       20

/* functions */

void bsp_cleanup( void );

m68k_isr_entry set_vector(
  rtems_isr_entry     handler,
  rtems_vector_number vector,
  int                 type
);

void console_init(void);

void Spurious_Initialize(void);

void _UART_flush(void);

void Clock_exit(void);

void outbyte(char);

#define NOINIT		0
#define RAW_INIT	1
#define RTEMS_INIT	2

void   _console_flush( unsigned chan );
int    _console_write( unsigned chan, const char* buf, int maxlen );
int    _console_read( unsigned chan, char* buf, int maxlen );
rtems_status_code  _console_init( unsigned chan, long baud, unsigned level );

#ifdef __cplusplus
}
#endif

#endif
/* end of include file */
