/*
 *
 *  This file contains the entry point for the application.
 *  The name of this entry point is compiler dependent.
 *  It jumps to the BSP which is responsible for performing
 *  all initialization.
 *
 *  Author:
 *    David Fiddes, D.J.Fiddes@hw.ac.uk
 *    http://www.calm.hw.ac.uk/davidf/coldfire/
 *
 *  COPYRIGHT (c) 1989-1998.
 *  On-Line Applications Research Corporation (OAR).
 *  Copyright assigned to U.S. Government, 1994.
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *
 *  http://www.OARcorp.com/rtems/license.html.
 *
 *  $Id: start5206.s,v 1.11 1998/02/17 23:44:51 joel Exp $
 */

#include "asm.h"

#define SRAMBAR 0x20000000

BEGIN_CODE

/***************************************************************************
   Function : Entry

   Description : Entry point to the system. In a raw system we would have
   put our vector table here but since we're coming from a monitor(dBUG)
   we have to fake it up. TODO: Add proper vector table for MCF5206.
 ***************************************************************************/
Entry:

/* Example minimal startup vector table - 
	The full one is left as an excercise for the reader.

	.long	#SRAMBAR		|   0: Initial SSP  (SBC5206-esq Internal SRAM location)
	.long	start			|   1: Initial PC

*/

	movel	#stack_init,a7		| set the real stack pointer (stack_init is set in linker script)
	jmp	SYM(start)		| Fake a jump to Initial PC


start:
	move.w	#0x2700,sr		| First turn off all interrupts!

	move.l	#SRAMBAR+1,d0		| Assign the MCF5206 internal SRAM...
	movec	d0,rambar0		| ...so we have a stack

	/*move.l	#SRAMBAR,sp		| Put the stack in SRAM*/
	
	/*
	 * Remainder of the startup code is handled by C code
	 */
	jmp	SYM(Init5206)		| Start C code (which never returns)
	
/***************************************************************************
   Function : CopyDataClearBSSAndStart

   Description : Copy DATA segment, clear BSS segment, initialize heap, 
   initialise real stack, start C program. Assume that DATA and BSS sizes 
   are multiples of 4.
 ***************************************************************************/
	PUBLIC (CopyDataClearBSSAndStart)
SYM(CopyDataClearBSSAndStart):
	lea	copy_start,a0		| Get start of DATA in RAM
	lea	SYM(etext),a2		| Get start of DATA in ROM
	cmpl	a0,a2			| Are they the same?
	beq.s	NOCOPY			| Yes, no copy necessary
	lea	copy_end,a1		| Get end of DATA in RAM
	bra.s	COPYLOOPTEST		| Branch into copy loop
COPYLOOP:
	movel	a2@+,a0@+		| Copy word from ROM to RAM
COPYLOOPTEST:
	cmpl	a1,a0			| Done?
	bcs.s	COPYLOOP		| No, skip
NOCOPY:

	lea	clear_start,a0		| Get start of BSS
	lea	clear_end,a1		| Get end of BSS
	clrl	d0			| Value to set
	bra.s	ZEROLOOPTEST		| Branch into clear loop
ZEROLOOP:
	movel	d0,a0@+			| Clear a word
ZEROLOOPTEST:
	cmpl	a1,a0			| Done?
	bcs.s	ZEROLOOP		| No, skip


	/*
	 * Right : Now we're ready to boot RTEMS
	 */
	jsr	SYM(boot_card)		| Call C boot_card function to startup RTEMS

        .align 2
	PUBLIC (_HeapSize)
SYM (_HeapSize):
        .long  HeapSize
	PUBLIC (_StackSize)
SYM (_StackSize):
        .long  StackSize
END_CODE

END

