/*
 * MCF5206 startup routines
 *
 *  Author:
 *    David Fiddes, D.J.Fiddes@hw.ac.uk
 *    http://www.calm.hw.ac.uk/davidf/coldfire/
 *
 *  COPYRIGHT (c) 1989-1998.
 *  On-Line Applications Research Corporation (OAR).
 *  Copyright assigned to U.S. Government, 1994.
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *
 *  http://www.OARcorp.com/rtems/license.html.
 * 
 *  $Id: init5206.c,v 1.13 1998/03/23 18:35:38 joel Exp $
 */

#include <rtems.h>
#include <bsp.h>

/* Global definition of the MCF5206 internal module pointer */
volatile SIM5206 *sim = (SIM5206 *)0x10000000;

#define m68k_set_cacr( _cacr ) \
	asm volatile ( "movec %0,%%cacr\n\t" \
				   "nop\n" \
				   : : "d" (_cacr) )

#define m68k_set_acr0( _acr0 ) \
	asm volatile (	"movec %0,%%acr0\n\t" \
					"nop\n\t" \
					: : "d" (_acr0) )

#define m68k_set_acr1( _acr1 ) \
	asm volatile (	"movec %0,%%acr1\n\t" \
					"nop\n\t" \
                   : : "d" (_acr1) )

#define m68k_set_srambar( _rambar0 ) \
	asm volatile (	"movec %0,%%rambar0\n\t" \
					"nop\n\t" \
                   : : "d" (_rambar0) )

#define m68k_set_mbar( _mbar ) \
	asm volatile (	"movec %0,%%mbar\n\t" \
					"nop\n\t" \
                   : : "d" (_mbar) )

#define mcf5206_enable_cache() \
	m68k_set_cacr( MCF5206_CACR_CENB )


#define mcf5206_disable_cache() \
	asm volatile (	" nop\n\t"		\
					" movec %0,%%cacr\n\t" \
					" nop\n\t" \
					" movec %0,%%cacr\n\t" \
					"nop\n\t" \
					: : "d" (MCF5206_CACR_CINV) )

	



/***************************************************************************
   Function : Init5206

   Description : This carries out basic low-level hardware setup needed to
   startup the MCF5206
 ***************************************************************************/
void Init5206 (void)
{
	extern void CopyDataClearBSSAndStart (void);
	
    m68k_set_vbr(0); /* Set the VBR to point to a monitor initialised table */

    /*
     * Invalidate the cache and disable it
     */
    mcf5206_disable_cache();

    /*
     * Setup ACRs so that if cache turned on, periphal accesses
     * are not messed up.  (Non-cacheable, serialized)
     */
    m68k_set_acr0 ( 0
        | MCF5206_ACR_BASE(0x30000000)
        | MCF5206_ACR_MASK(0x0F000000)
        | MCF5206_ACR_EN
        | MCF5206_ACR_S_IGNORE
    );
    m68k_set_acr1 ( 0
        | MCF5206_ACR_BASE(0x30000000)
        | MCF5206_ACR_MASK(0x0F000000)
        | MCF5206_ACR_EN
        | MCF5206_ACR_S_IGNORE
    );

	mcf5206_enable_cache(); 
	
	/*
	 * Copy data, clear BSS, switch stacks and call boot_card()
	 */
	CopyDataClearBSSAndStart ();
}
